@@ -1,3 +1,9 @@
+/*
+ * smartcalc v1.0.2
+ * Copyright (c) Erhan BARIS (Ruslan Ognyanov Asenov)
+ * Licensed under the GNU General Public License v2.0.
+ */
+
 use alloc::sync::Arc;
 use alloc::vec::Vec;
 use core::cell::RefCell;
@@ -8,20 +14,19 @@ use alloc::borrow::ToOwned;
 use alloc::string::String;
 use alloc::format;
 use core::ops::Deref;
+use chrono::{NaiveDateTime, TimeZone};
 
 use serde_derive::{Deserialize, Serialize};
 use alloc::collections::btree_map::BTreeMap;
-use chrono::{Duration, NaiveDate, NaiveDateTime, NaiveTime};
+use chrono::{Duration, NaiveDate};
 use crate::compiler::DataItem;
 use crate::config::SmartCalcConfig;
 use crate::token::ui_token::{UiTokenType};
 
 use crate::tokinizer::{TokenInfo, TokenInfoStatus, Tokinizer};
 
-pub type TokinizeResult     = Result<Vec<TokenInfo>, (&'static str, u16, u16)>;
 pub type ExpressionFunc     = fn(config: &SmartCalcConfig, tokinizer: &Tokinizer, fields: &BTreeMap<String, Arc<TokenInfo>>) -> core::result::Result<TokenType, String>;
-pub type TokenParserResult  = Result<bool, (&'static str, u16)>;
-pub type AstResult          = Result<BramaAstType, (&'static str, u16, u16)>;
+pub type AstResult          = Result<SmartCalcAstType, (&'static str, u16, u16)>;
 
 pub struct Money(pub f64, pub Arc<CurrencyInfo>);
 impl Money {
@@ -39,7 +44,7 @@ pub struct VariableInfo {
     pub index: usize,
     pub name: String,
     pub tokens: Vec<Rc<TokenType>>,
-    pub data: RefCell<Rc<BramaAstType>>
+    pub data: RefCell<Rc<SmartCalcAstType>>
 }
 
 impl PartialEq for VariableInfo {
@@ -72,7 +77,8 @@ pub enum FieldType {
     TypeGroup(Vec<String>, String),
     Month(String),
     Duration(String),
-    Memory(String)
+    Memory(String),
+    Timezone(String)
 }
 
 unsafe impl Send for FieldType {}
@@ -91,20 +97,12 @@ impl FieldType {
             FieldType::TypeGroup(_, _) => "TYPE_GROUP".to_string(),
             FieldType::Month(_) => "MONTH".to_string(),
             FieldType::Duration(_) => "DURATION".to_string(),
-            FieldType::Memory(_) => "MEMORY".to_string()
+            FieldType::Memory(_) => "MEMORY".to_string(),
+            FieldType::Timezone(_) => "TIMEZONE".to_string()
         }
     }
 }
 
-#[repr(C)]
-#[derive(Clone)]
-#[derive(Debug)]
-pub enum BramaNumberSystem {
-    Binary      = 0,
-    Octal       = 1,
-    Decimal     = 2,
-    Hexadecimal = 3
-}
 
 #[derive(Clone)]
 #[derive(Debug)]
@@ -154,13 +152,19 @@ impl Ord for CurrencyInfo {
     }
 }
 
+#[derive(Debug, Clone, PartialEq)]
+pub struct TimeOffset {
+    pub name: String,
+    pub offset: i32
+}
+
 #[derive(Debug, Clone)]
 pub enum TokenType {
     Number(f64),
     Text(String),
-    Time(NaiveTime),
-    Date(NaiveDate),
-    DateTime(NaiveDateTime),
+    Time(NaiveDateTime, TimeOffset),
+    Date(NaiveDate, TimeOffset),
+    DateTime(NaiveDateTime, TimeOffset),
     Operator(char),
     Field(Rc<FieldType>),
     Percent(f64),
@@ -168,13 +172,15 @@ pub enum TokenType {
     Variable(Rc<VariableInfo>),
     Month(u32),
     Duration(Duration),
-    Memory(f64, MemoryType)
+    Memory(f64, MemoryType),
+    Timezone(String, i32)
 }
 
 
 impl PartialEq for TokenType {
     fn eq(&self, other: &Self) -> bool {
         match (&self, &other) {
+            (TokenType::Timezone(l_value, l_type),     TokenType::Timezone(r_value, r_type)) => *l_value == *r_value && *l_type == *r_type,
             (TokenType::Memory(l_value, l_type),     TokenType::Memory(r_value, r_type)) => *l_value == *r_value && *l_type == *r_type,
             (TokenType::Text(l_value),     TokenType::Text(r_value)) => *l_value == *r_value,
             (TokenType::Number(l_value),   TokenType::Number(r_value)) => l_value == r_value,
@@ -182,12 +188,13 @@ impl PartialEq for TokenType {
             (TokenType::Operator(l_value), TokenType::Operator(r_value)) => l_value == r_value,
             (TokenType::Variable(l_value), TokenType::Variable(r_value)) => l_value == r_value,
             (TokenType::Money(l_value, l_symbol), TokenType::Money(r_value, r_symbol)) => l_value == r_value && l_symbol == r_symbol,
-            (TokenType::Time(l_value),     TokenType::Time(r_value)) => l_value == r_value,
+            (TokenType::Time(l_value, l_tz),     TokenType::Time(r_value, r_tz)) => l_value == r_value && l_tz == r_tz,
             (TokenType::Month(l_value),     TokenType::Month(r_value)) => l_value == r_value,
             (TokenType::Duration(l_value),     TokenType::Duration(r_value)) => l_value == r_value,
-            (TokenType::Date(l_value),     TokenType::Date(r_value)) => l_value == r_value,
+            (TokenType::Date(l_value, l_tz),     TokenType::Date(r_value, r_tz)) => l_value == r_value && l_tz == r_tz,
             (TokenType::Field(l_value),    TokenType::Field(r_value)) => {
                 match (l_value.deref(), r_value.deref()) {
+                    (FieldType::Timezone(l), FieldType::Timezone(r)) => r == l,
                     (FieldType::Memory(l), FieldType::Memory(r)) => r == l,
                     (FieldType::Percent(l), FieldType::Percent(r)) => r == l,
                     (FieldType::Number(l),  FieldType::Number(r)) => r == l,
@@ -212,9 +219,21 @@ impl ToString for TokenType {
         match &self {
             TokenType::Number(number) => number.to_string(),
             TokenType::Text(text) => text.to_string(),
-            TokenType::Time(time) => time.to_string(),
-            TokenType::Date(date) => date.to_string(),
-            TokenType::DateTime(datetime) => datetime.to_string(),
+            TokenType::Time(time, tz) => {
+                let tz_offset = chrono::FixedOffset::east(tz.offset * 60);
+                let datetime = tz_offset.from_utc_datetime(&time);
+                alloc::format!("{} {}", datetime.format("%H:%M:%S").to_string(), tz.name)
+            },
+            TokenType::Date(date, tz) => {
+                let tz_offset = chrono::FixedOffset::east(tz.offset * 60);
+                let datetime = tz_offset.from_utc_date(&date);
+                alloc::format!("{} {}", datetime.format("%d/%m/%Y").to_string(), tz.name)
+            },
+            TokenType::DateTime(datetime, tz) => {
+                let tz_offset = chrono::FixedOffset::east(tz.offset * 60);
+                let datetime = tz_offset.from_utc_datetime(&datetime);
+                alloc::format!("{} {}", datetime.format("%d/%m/%Y %H:%M:%S").to_string(), tz.name)
+            },
             TokenType::Operator(ch) => ch.to_string(),
             TokenType::Field(_) => "field".to_string(),
             TokenType::Percent(number) => format!("%{}", number),
@@ -223,6 +242,7 @@ impl ToString for TokenType {
             TokenType::Month(month) => month.to_string(),
             TokenType::Duration(duration) => duration.to_string(),
             TokenType::Memory(memory, memory_type) => format!("{} {:?}", memory.to_string(), memory_type),
+            TokenType::Timezone(timezone, offset) => format!("{} {:?}", timezone, offset)
         }
     }
 }
@@ -233,9 +253,9 @@ impl TokenType {
         match self {
             TokenType::Number(_) => "NUMBER".to_string(),
             TokenType::Text(_) => "TEXT".to_string(),
-            TokenType::Time(_) => "TIME".to_string(),
-            TokenType::Date(_) => "DATE".to_string(),
-            TokenType::DateTime(_) => "DATE_TIME".to_string(),
+            TokenType::Time(_, _) => "TIME".to_string(),
+            TokenType::Date(_, _) => "DATE".to_string(),
+            TokenType::DateTime(_, _) => "DATE_TIME".to_string(),
             TokenType::Operator(_) => "OPERATOR".to_string(),
             TokenType::Field(_) => "FIELD".to_string(),
             TokenType::Percent(_) => "PERCENT".to_string(),
@@ -243,32 +263,35 @@ impl TokenType {
             TokenType::Variable(_) => "VARIABLE".to_string(),
             TokenType::Month(_) => "MONTH".to_string(),
             TokenType::Duration(_) => "DURATION".to_string(),
-            TokenType::Memory(_, _) => "MEMORY".to_string()
+            TokenType::Memory(_, _) => "MEMORY".to_string(),
+            TokenType::Timezone(_, _) => "TIMEZONE".to_string()
         }
     }
 
-    pub fn variable_compare(left: &TokenInfo, right: Rc<BramaAstType>) -> bool {
+    pub fn variable_compare(left: &TokenInfo, right: Rc<SmartCalcAstType>) -> bool {
         match &left.token_type.borrow().deref() {
             Some(token) => match (&token, right.deref()) {
-                (TokenType::Text(l_value), BramaAstType::Symbol(r_value)) => l_value.deref() == r_value,
-                (TokenType::Memory(l_value, l_type), BramaAstType::Item(r_value)) => r_value.is_same(&(l_value.clone(), l_type.clone())),
-                (TokenType::Number(l_value), BramaAstType::Item(r_value)) => r_value.is_same(l_value),
-                (TokenType::Percent(l_value), BramaAstType::Item(r_value)) => r_value.is_same(l_value),
-                (TokenType::Duration(l_value), BramaAstType::Item(r_value)) => r_value.is_same(l_value),
-                (TokenType::Time(l_value), BramaAstType::Item(r_value)) => r_value.is_same(l_value),
-                (TokenType::Money(l_value, l_symbol), BramaAstType::Item(r_value)) => r_value.is_same(&(l_value.clone(), l_symbol.clone())),
-                (TokenType::Date(l_value), BramaAstType::Item(r_value)) => r_value.is_same(l_value),
+                (TokenType::Text(l_value), SmartCalcAstType::Symbol(r_value)) => l_value.deref() == r_value,
+                (TokenType::Memory(l_value, l_type), SmartCalcAstType::Item(r_value)) => r_value.is_same(&(l_value.clone(), l_type.clone())),
+                (TokenType::Timezone(l_value, l_type), SmartCalcAstType::Item(r_value)) => r_value.is_same(&(l_value.clone(), l_type.clone())),
+                (TokenType::Number(l_value), SmartCalcAstType::Item(r_value)) => r_value.is_same(l_value),
+                (TokenType::Percent(l_value), SmartCalcAstType::Item(r_value)) => r_value.is_same(l_value),
+                (TokenType::Duration(l_value), SmartCalcAstType::Item(r_value)) => r_value.is_same(l_value),
+                (TokenType::Time(l_value, l_tz), SmartCalcAstType::Item(r_value)) => r_value.is_same(&(l_value.clone(), l_tz.clone())),
+                (TokenType::Money(l_value, l_symbol), SmartCalcAstType::Item(r_value)) => r_value.is_same(&(l_value.clone(), l_symbol.clone())),
+                (TokenType::Date(l_value, l_tz), SmartCalcAstType::Item(r_value)) => r_value.is_same(&(l_value.clone(), l_tz.clone())),
                 (TokenType::Field(l_value), _) => {
                     match (l_value.deref(), right.deref()) {
-                        (FieldType::Percent(_), BramaAstType::Item(item)) => item.type_name() == "PERCENT",
-                        (FieldType::Number(_), BramaAstType::Item(item)) => item.type_name() == "NUMBER",
-                        (FieldType::Text(_), BramaAstType::Symbol(_)) => true,
-                        (FieldType::Time(_), BramaAstType::Item(item)) => item.type_name() == "TIME",
-                        (FieldType::Money(_),   BramaAstType::Item(item)) => item.type_name() == "MONEY",
-                        (FieldType::Month(_),   BramaAstType::Month(_)) => true,
-                        (FieldType::Duration(_),   BramaAstType::Item(item)) => item.type_name() == "DURATION",
-                        (FieldType::Memory(_),   BramaAstType::Item(item)) => item.type_name() == "MEMORY",
-                        (FieldType::Date(_),   BramaAstType::Item(item)) => item.type_name() == "DATE",
+                        (FieldType::Percent(_), SmartCalcAstType::Item(item)) => item.type_name() == "PERCENT",
+                        (FieldType::Number(_), SmartCalcAstType::Item(item)) => item.type_name() == "NUMBER",
+                        (FieldType::Text(_), SmartCalcAstType::Symbol(_)) => true,
+                        (FieldType::Time(_), SmartCalcAstType::Item(item)) => item.type_name() == "TIME",
+                        (FieldType::Money(_),   SmartCalcAstType::Item(item)) => item.type_name() == "MONEY",
+                        (FieldType::Month(_),   SmartCalcAstType::Month(_)) => true,
+                        (FieldType::Duration(_),   SmartCalcAstType::Item(item)) => item.type_name() == "DURATION",
+                        (FieldType::Memory(_),   SmartCalcAstType::Item(item)) => item.type_name() == "MEMORY",
+                        (FieldType::Timezone(_),   SmartCalcAstType::Item(item)) => item.type_name() == "TIMEZONE",
+                        (FieldType::Date(_),   SmartCalcAstType::Item(item)) => item.type_name() == "DATE",
                         (FieldType::TypeGroup(types, _), right_ast) => types.contains(&right_ast.type_name()),
                         (_, _) => false,
                     }
@@ -292,7 +315,8 @@ impl TokenType {
                 FieldType::Duration(field_name)  => Some(field_name.to_string()),
                 FieldType::Group(field_name, _)  => Some(field_name.to_string()),
                 FieldType::TypeGroup(_, field_name) => Some(field_name.to_string()),
-                FieldType::Memory(field_name) => Some(field_name.to_string())
+                FieldType::Memory(field_name) => Some(field_name.to_string()),
+                FieldType::Timezone(field_name) => Some(field_name.to_string())
             },
             _ => None
         }
@@ -427,20 +451,22 @@ impl core::cmp::PartialEq<TokenType> for TokenInfo {
                 (TokenType::Number(l_value),   TokenType::Number(r_value)) => l_value == r_value,
                 (TokenType::Percent(l_value),  TokenType::Percent(r_value)) => l_value == r_value,
                 (TokenType::Operator(l_value), TokenType::Operator(r_value)) => l_value == r_value,
-                (TokenType::Date(l_value), TokenType::Date(r_value)) => l_value == r_value,
+                (TokenType::Date(l_value, l_tz), TokenType::Date(r_value, r_tz)) => l_value == r_value && l_tz == r_tz,
                 (TokenType::Duration(l_value), TokenType::Duration(r_value)) => l_value == r_value,
                 (TokenType::Month(l_value), TokenType::Month(r_value)) => l_value == r_value,
                 (TokenType::Money(l_value, l_symbol), TokenType::Money(r_value, r_symbol)) => l_value == r_value && l_symbol == r_symbol,
                 (TokenType::Memory(l_value, l_symbol), TokenType::Memory(r_value, r_symbol)) => l_value == r_value && l_symbol == r_symbol,
+                (TokenType::Timezone(l_value, l_symbol), TokenType::Timezone(r_value, r_symbol)) => l_value == r_value && l_symbol == r_symbol,
                 (TokenType::Variable(l_value), TokenType::Variable(r_value)) => l_value == r_value,
                 (TokenType::Field(l_value), _) => {
                     match (l_value.deref(), &other) {
                         (FieldType::Percent(_), TokenType::Percent(_)) => true,
                         (FieldType::Memory(_),  TokenType::Memory(_, _)) => true,
+                        (FieldType::Timezone(_),  TokenType::Timezone(_, _)) => true,
                         (FieldType::Number(_),  TokenType::Number(_)) => true,
                         (FieldType::Text(_),    TokenType::Text(_) ) => true,
-                        (FieldType::Time(_),    TokenType::Time(_)) => true,
-                        (FieldType::Date(_),    TokenType::Date(_)) => true,
+                        (FieldType::Time(_),    TokenType::Time(_, _)) => true,
+                        (FieldType::Date(_),    TokenType::Date(_, _)) => true,
                         (FieldType::Money(_),   TokenType::Money(_, _)) => true,
                         (FieldType::Month(_),   TokenType::Month(_)) => true,
                         (FieldType::Duration(_),   TokenType::Duration(_)) => true,
@@ -453,10 +479,11 @@ impl core::cmp::PartialEq<TokenType> for TokenInfo {
                     match (r_value.deref(), &l_token) {
                         (FieldType::Percent(_), TokenType::Percent(_)) => true,
                         (FieldType::Memory(_), TokenType::Memory(_, _)) => true,
+                        (FieldType::Timezone(_), TokenType::Timezone(_, _)) => true,
                         (FieldType::Number(_),  TokenType::Number(_)) => true,
                         (FieldType::Text(_),    TokenType::Text(_) ) => true,
-                        (FieldType::Time(_),    TokenType::Time(_)) => true,
-                        (FieldType::Date(_),    TokenType::Date(_)) => true,
+                        (FieldType::Time(_),    TokenType::Time(_, _)) => true,
+                        (FieldType::Date(_),    TokenType::Date(_, _)) => true,
                         (FieldType::Money(_),   TokenType::Money(_, _)) => true,
                         (FieldType::Duration(_),   TokenType::Duration(_)) => true,
                         (FieldType::Month(_),   TokenType::Month(_)) => true,
@@ -484,19 +511,21 @@ impl PartialEq for TokenInfo {
                 (TokenType::Number(l_value),   TokenType::Number(r_value)) => l_value == r_value,
                 (TokenType::Percent(l_value),  TokenType::Percent(r_value)) => l_value == r_value,
                 (TokenType::Operator(l_value), TokenType::Operator(r_value)) => l_value == r_value,
-                (TokenType::Date(l_value), TokenType::Date(r_value)) => l_value == r_value,
+                (TokenType::Date(l_value, l_tz), TokenType::Date(r_value, r_tz)) => l_value == r_value && l_tz == r_tz,
                 (TokenType::Duration(l_value), TokenType::Duration(r_value)) => l_value == r_value,
                 (TokenType::Money(l_value, l_symbol), TokenType::Money(r_value, r_symbol)) => l_value == r_value && l_symbol == r_symbol,
                 (TokenType::Memory(l_value, l_symbol), TokenType::Memory(r_value, r_symbol)) => l_value == r_value && l_symbol == r_symbol,
+                (TokenType::Timezone(l_value, l_symbol), TokenType::Timezone(r_value, r_symbol)) => l_value == r_value && l_symbol == r_symbol,
                 (TokenType::Variable(l_value), TokenType::Variable(r_value)) => l_value == r_value,
                 (TokenType::Field(l_value), _) => {
                     match (l_value.deref(), &r_token) {
                         (FieldType::Percent(_), TokenType::Percent(_)) => true,
                         (FieldType::Memory(_), TokenType::Memory(_, _)) => true,
+                        (FieldType::Timezone(_), TokenType::Timezone(_, _)) => true,
                         (FieldType::Number(_),  TokenType::Number(_)) => true,
                         (FieldType::Text(_),    TokenType::Text(_) ) => true,
-                        (FieldType::Time(_),    TokenType::Time(_)) => true,
-                        (FieldType::Date(_),    TokenType::Date(_)) => true,
+                        (FieldType::Time(_),    TokenType::Time(_, _)) => true,
+                        (FieldType::Date(_),    TokenType::Date(_, _)) => true,
                         (FieldType::Money(_),   TokenType::Money(_, _)) => true,
                         (FieldType::Duration(_), TokenType::Duration(_)) => true,
                         (FieldType::Month(_),   TokenType::Month(_)) => true,
@@ -509,10 +538,11 @@ impl PartialEq for TokenInfo {
                     match (r_value.deref(), &l_token) {
                         (FieldType::Percent(_), TokenType::Percent(_)) => true,
                         (FieldType::Memory(_), TokenType::Memory(_, _)) => true,
+                        (FieldType::Timezone(_), TokenType::Timezone(_, _)) => true,
                         (FieldType::Number(_),  TokenType::Number(_)) => true,
                         (FieldType::Text(_),    TokenType::Text(_) ) => true,
-                        (FieldType::Time(_),    TokenType::Time(_)) => true,
-                        (FieldType::Date(_),    TokenType::Date(_)) => true,
+                        (FieldType::Time(_),    TokenType::Time(_, _)) => true,
+                        (FieldType::Date(_),    TokenType::Date(_, _)) => true,
                         (FieldType::Money(_),   TokenType::Money(_, _)) => true,
                         (FieldType::Month(_),   TokenType::Month(_)) => true,
                         (FieldType::Duration(_), TokenType::Duration(_)) => true,
@@ -584,44 +614,44 @@ impl MemoryType {
 #[repr(C)]
 #[derive(Clone)]
 #[derive(Debug)]
-pub enum BramaAstType {
+pub enum SmartCalcAstType {
     None,
     Field(Rc<FieldType>),
     Item(Arc<dyn DataItem>),
     Month(u32),
     Binary {
-        left: Rc<BramaAstType>,
+        left: Rc<SmartCalcAstType>,
         operator: char,
-        right: Rc<BramaAstType>
+        right: Rc<SmartCalcAstType>
     },
-    PrefixUnary(char, Rc<BramaAstType>),
+    PrefixUnary(char, Rc<SmartCalcAstType>),
     Assignment {
         index: usize,
-        expression: Rc<BramaAstType>
+        expression: Rc<SmartCalcAstType>
     },
     Symbol(String),
     Variable(Rc<VariableInfo>)
 }
 
-impl BramaAstType {
+impl SmartCalcAstType {
     pub fn type_name(&self) -> String {
         match self {
-            BramaAstType::None => "NONE".to_string(),
-            BramaAstType::Item(_) => "ITEM".to_string(),
-            BramaAstType::Field(_) => "FIELD".to_string(),
-            BramaAstType::Month(_) => "MONTH".to_string(),
-            BramaAstType::Binary {
+            SmartCalcAstType::None => "NONE".to_string(),
+            SmartCalcAstType::Item(_) => "ITEM".to_string(),
+            SmartCalcAstType::Field(_) => "FIELD".to_string(),
+            SmartCalcAstType::Month(_) => "MONTH".to_string(),
+            SmartCalcAstType::Binary {
                 left: _,
                 operator: _,
                 right: _
             } => "BINARY".to_string(),
-            BramaAstType::PrefixUnary(_, _) => "PREFIX_UNARY".to_string(),
-            BramaAstType::Assignment {
+            SmartCalcAstType::PrefixUnary(_, _) => "PREFIX_UNARY".to_string(),
+            SmartCalcAstType::Assignment {
                 index: _,
                 expression: _
             } => "ASSIGNMENT".to_string(),
-            BramaAstType::Symbol(_) => "SYMBOL".to_string(),
-            BramaAstType::Variable(_) => "VARIABLE".to_string()
+            SmartCalcAstType::Symbol(_) => "SYMBOL".to_string(),
+            SmartCalcAstType::Variable(_) => "VARIABLE".to_string()
         }
     }
 }
